local Default = {
	anchor = "UIParent",
	anchorPoint = "TOP",
	anchorTo = "TOP",
	xOffset = 0,
	yOffset = -80,
	fontSize = 14,
	size = 40,
	textXOffset = 0,
	textYOffset = -8,
	showHarm = true,
	showHeal = true,
	drag = false,
	locked = true,
	pos = {},
	range1 = 0,
	range1Texture = nil,
	range1Text = Range1,
	range2 = 0,
	range2Texture = nil,
	range2Text = Range2,
	range3 = 0,
	range3Texture = nil,
	range3Text = Range3,
	range4 = 0,
	range4Texture = nil,
	range4Text = Range4,
	range5 = 0,
	range5Texture = nil,
	range5Text = Range5,
	close = false,	
	toggle = false,
	closeTexture = nil,
	heal1 = 0,
	heal1Texture = nil,
	heal1Text = Heal1,
	heal2 = 0,
	heal2Texture = nil,
	heal2Text = Heal2,
	combat = true,
	iconAlpha = 1,
	textAlpha = 1
}

Ranger = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

function Ranger:OnEnable()
	self:CreateFrameWork()
 	self:RegisterEvent("PLAYER_TARGET_CHANGED", "TargetChanged")
	self:RegisterEvent("UNIT_FACTION", "TargetChanged")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat")
	rangerCombat = InCombatLockdown()
	self:TargetChanged()
	rangerEnabled = true
end

function Ranger:OnDisable()
	RangerFrame:Hide()
 	self:UnregisterAllEvents()
	rangerEnabled = false
end

function Ranger:EnterCombat()
	rangerCombat = true
	Ranger:SetRange()
end

function Ranger:LeaveCombat()
	rangerCombat = false
	Ranger:SetRange()
end

function Ranger:OnInitialize()
	local Options = {
		type = "group",
		args = {
			position = {
				type = "group",
				desc = "Position/size options.",
				name = "Position",
				args = {
					anchor = {
						name = "Anchor", type = "text",
						desc = "Set the frame the display anchors to.",
						get = function() return self.db.profile.anchor end,
						set = function( v ) self.db.profile.anchor = v self:Layout() end,
						usage = "<anchor frame>",
					},
					anchorPoint = {
						name = "AnchorPoint", type = "text",
						desc = "Set the point the display is anchored by.",
						get = function() return self.db.profile.anchorPoint end,
						set = function( v ) self.db.profile.anchorPoint = v self:Layout() end,
						usage = "<point frame anchors by>",
						validate = {"TOP","TOPRIGHT","RIGHT","BOTTOMRIGHT","BOTTOM","BOTTOMLEFT","LEFT","TOPLEFT","CENTER"},
					},
					anchorTo = {
						name = "AnchorTo", type = "text",
						desc = "Set the point the display anchors to.",
						get = function() return self.db.profile.anchorTo end,
						set = function( v ) self.db.profile.anchorTo = v self:Layout() end,
						usage = "<point to anchor to>",
						validate = {"TOP","TOPRIGHT","RIGHT","BOTTOMRIGHT","BOTTOM","BOTTOMLEFT","LEFT","TOPLEFT","CENTER"},
					},
					xOffset = {
						name = "xOffset", type = "range", min = -1000, max = 1000, step = 1,
						desc = "Set the horizontal offset.",
						get = function() return self.db.profile.xOffset end,
						set = function( v ) self.db.profile.xOffset = v self:Layout() end,
					},
					yOffset = {
						name = "yOffset", type = "range", min = -1000, max = 1000, step = 1,
						desc = "Set the vertical offset.",
						get = function() return self.db.profile.yOffset end,
						set = function( v ) self.db.profile.yOffset = v self:Layout() end,
					},
					size = {
						name = "Size", type = "range", min = 10, max = 100, step = 1,
						desc = "Set the icon size.",
						get = function() return self.db.profile.size end,
						set = function( v ) self.db.profile.size = v self:Layout() end,
					},
					iconAlpha = {
						name = "IconAlpha", type = "range", min = 0, max = 1, step = 0.01,
						desc = "Set the icon alpha.",
						get = function() return self.db.profile.iconAlpha end,
						set = function( v ) self.db.profile.iconAlpha = v self:Layout() end,
					},
					drag = {
						name = "Drag", type = "toggle",
						desc = "Overrides the parent/offset settings and makes the frame draggable when unlocked.",
						get = function() return self.db.profile.drag end,
						set = function( v ) self.db.profile.drag = not self.db.profile.drag self:Layout() end,
					},
					lock = {
						name = "Lock", type = "toggle",
						desc = "Locks/unlocks frame dragging, only relevant when drag toggle is true.",
						get = function() return self.db.profile.lock end,
						set = function( v ) self.db.profile.lock = not self.db.profile.lock self:Layout() end,
					},
					reset = {
						name = "Lock", type = "execute",
						desc = "Resets position to default values.",
						func = function() self.db.profile.xOffset = 0 self.db.profile.yOffset = -80 self.db.profile.anchor = "UIParent" self.db.profile.anchorTo = "TOP" self.db.profile.anchorPoint = "TOP" self.db.profile.pos = {} self:Layout() end,
					},
					
				},
			},
			text = {
				type = "group",
				desc = "Text options.",
				name = "Text",
				args = {
					fontSize = {
						name = "FontSize", type = "range", min = 6, max = 32, step = 1,
						desc = "Set the font size.",
						get = function() return self.db.profile.fontSize end,
						set = function( v ) self.db.profile.fontSize = v self:Layout() end,
					},
					textXOffset = {
						name = "xOffset", type = "range", min = -50, max = 50, step = 1,
						desc = "Set the horizontal offset of the text.",
						get = function() return self.db.profile.textXOffset end,
						set = function( v ) self.db.profile.textXOffset = v self:Layout() end,
					},
					textYOffset = {
						name = "yOffset", type = "range", min = -50, max = 50, step = 1,
						desc = "Set the vertical offset of the text.",
						get = function() return self.db.profile.textYOffset end,
						set = function( v ) self.db.profile.textYOffset = v self:Layout() end,
					},
					textAlpha = {
						name = "TextAlpha", type = "range", min = 0, max = 1, step = 0.01,
						desc = "Set the text alpha.",
						get = function() return self.db.profile.textAlpha end,
						set = function( v ) self.db.profile.textAlpha = v self:Layout() end,
					},
				},
			},
			advanced = {
				type = "group",
				desc = "Advanced options for offensive spells.",
				name = "Advanced",
				args = {
					toggle = {
						name = "Toggle", type = "toggle",
						desc = "Toggles advanced mode. This will override the class specific default ranges. Ranges take valid action id's. Set closest range on range1 and so forth. Attention: This is NOT meant to be easy to configure, hence the label.",
						get = function() return self.db.profile.toggle end,
						set = function( v ) self.db.profile.toggle = not self.db.profile.toggle self:TargetChanged() end,
					},
					range1 = {
						name = "Range1", type = "range", min = 0, max = 120, step = 1,
						desc = "Sets the action ID for range 1, 0 to clear.",
						get = function() return self.db.profile.range1 end,
						set = function( v ) self.db.profile.range1 = v self:TargetChanged() end,
					},
					range1Texture = {
						name = "Range1Texture", type = "text",
						desc = "Set the texture for range 1. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use button texture for given id if empty, nil to clear.",
						get = function() return self.db.profile.range1Texture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range1Texture = nil else self.db.profile.range1Texture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",
					},
					range1Text = {
						name = "Range1Text", type = "text",
						desc = "Set the text for range 1. Nil to clear.",
						get = function() return self.db.profile.range1Text end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range1Text = nil else self.db.profile.range1Text = v end self:TargetChanged() end,
						usage = "<text>",
					},
					range2 = {
						name = "Range2", type = "range", min = 0, max = 120, step = 1,
						desc = "Sets the action ID for range 2, 0 to clear.",
						get = function() return self.db.profile.range2 end,
						set = function( v ) self.db.profile.range2 = v self:TargetChanged() end,
					},
					range2Texture = {
						name = "Range2Texture", type = "text",
						desc = "Set the texture for range 2. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use button texture for given id if empty, nil to clear.",
						get = function() return self.db.profile.range2Texture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range2Texture = nil else self.db.profile.range2Texture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",
					},
					range2Text = {
						name = "Range2Text", type = "text",
						desc = "Set the text for range 2. Nil to clear.",
						get = function() return self.db.profile.range2Text end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range2Text = nil else self.db.profile.range2Text = v end self:TargetChanged() end,
						usage = "<text>",
					},
					range3 = {
						name = "Range3", type = "range", min = 0, max = 120, step = 1,
						desc = "Sets the action ID for range 3, 0 to clear.",
						get = function() return self.db.profile.range3 end,
						set = function( v ) self.db.profile.range3 = v self:TargetChanged() end,
					},
					range3Text = {
						name = "Range3Text", type = "text",
						desc = "Set the text for range 3. Nil to clear.",
						get = function() return self.db.profile.range3Text end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range3Text = nil else self.db.profile.range3Text = v end self:TargetChanged() end,
						usage = "<text>",
					},
					range3Texture = {
						name = "Range3Texture", type = "text",
						desc = "Set the texture for range 3. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use button texture for given id if empty, nil to clear.",
						get = function() return self.db.profile.range3Texture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range3Texture = nil else self.db.profile.range3Texture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",
					},
					range4 = {
						name = "Range4", type = "range", min = 0, max = 120, step = 1,
						desc = "Sets the action ID for range 1, 0 to clear.",
						get = function() return self.db.profile.range4 end,
						set = function( v ) self.db.profile.range4 = v self:TargetChanged() end,
					},
					range4Texture = {
						name = "Range4Texture", type = "text",
						desc = "Set the texture for range 4. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use button texture for given id if empty, nil to clear.",
						get = function() return self.db.profile.range4Texture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range4Texture = nil else self.db.profile.range4Texture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",
					},
					range4Text = {
						name = "Range4Text", type = "text",
						desc = "Set the text for range 4. Nil to clear.",
						get = function() return self.db.profile.range4Text end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range4Text = nil else self.db.profile.range4Text = v end self:TargetChanged() end,
						usage = "<text>",
					},
					range5 = {
						name = "Range5", type = "range", min = 0, max = 120, step = 1,
						desc = "Sets the action ID for range 5, 0 to clear.",
						get = function() return self.db.profile.range5 end,
						set = function( v ) self.db.profile.range5 = v self:TargetChanged() end,
					},
					range5Texture = {
						name = "Range5Texture", type = "text",
						desc = "Set the texture for range 5. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use button texture for given id if empty, nil to clear.",
						get = function() return self.db.profile.range5Texture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range5Texture = nil else self.db.profile.range5Texture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",
					},
					range5Text = {
						name = "Range5Text", type = "text",
						desc = "Set the text for range 5. Nil to clear.",
						get = function() return self.db.profile.range5Text end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.range5Text = nil else self.db.profile.range5Text = v end self:TargetChanged() end,
						usage = "<text>",
					},
					close = {
						name = "Close", type = "toggle",
						desc = "Toggles 10 yard range display. Will break any shorter ranges.",
						get = function() return self.db.profile.close end,
						set = function( v ) self.db.profile.close = not self.db.profile.close self:TargetChanged() end,
					},
					closeTexture = {
						name = "CloseTexture", type = "text",
						desc = "Set the texture for 10 yard range. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use faerie fire icon if empty, nil to clear.",
						get = function() return self.db.profile.closeTexture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.closeTexture = nil else self.db.profile.closeTexture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",
					},
					heal1 = {
						name = "Heal1", type = "range", min = 0, max = 120, step = 1,
						desc = "Sets the action ID for heal 1, 0 to clear.",
						get = function() return self.db.profile.heal1 end,
						set = function( v ) self.db.profile.heal1 = v self:TargetChanged() end,
					},
					heal1Texture = {
						name = "Heal1Texture", type = "text",
						desc = "Set the texture for heal 1. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use button texture for given id if empty, nil to clear.",
						get = function() return self.db.profile.heal1Texture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.heal1Texture = nil else self.db.profile.heal1Texture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",                                                                                              
					},
					heal1Text = {
						name = "Heal1Text", type = "text",
						desc = "Set the text for heal 1. Nil to clear.",
						get = function() return self.db.profile.heal1Text end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.heal1Text = nil else self.db.profile.heal1Text = v end self:TargetChanged() end,
						usage = "<text>",
					},
					heal2 = {
						name = "Heal2", type = "range", min = 0, max = 120, step = 1,
						desc = "Sets the action ID for heal 2, 0 to clear.",
						get = function() return self.db.profile.heal2 end,
						set = function( v ) self.db.profile.heal2 = v self:TargetChanged() end,
					},
					heal2Texture = {
						name = "Heal2Texture", type = "text",
						desc = "Set the texture for heal 2. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use button texture for given id if empty, nil to clear.",
						get = function() return self.db.profile.heal2Texture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.heal2Texture = nil else self.db.profile.heal2Texture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",
					},
					heal2Text = {
						name = "Heal2Text", type = "text",
						desc = "Set the text for heal 2. Nil to clear.",
						get = function() return self.db.profile.heal2Text end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.heal2Text = nil else self.db.profile.heal2Text = v end self:TargetChanged() end,
						usage = "<text>",
					},
					oorTexture = {
						name = "OorTexture", type = "text",
						desc = "Set the texture for out of range. Takes valid icon textures (will append Interface\\Icons\\ to given value). Will use tranquility texture if empty, nil to clear.",
						get = function() return self.db.profile.oorTexture end,
						set = function( v ) if v == "nil" or v == "0" then self.db.profile.oorTexture = nil else self.db.profile.oorTexture = v end self:TargetChanged() end,
						usage = "Interface\\Icons\\<texture name>",
					},
				},
			},
			showHarm = {
				name = "ShowHarm", type = "toggle",
				desc = "Toggles display on hostile targets.",
				get = function() return self.db.profile.showHarm end,
				set = function( v ) self.db.profile.showHarm = not self.db.profile.showHarm self:TargetChanged() end,
			},
			showHeal = {
				name = "ShowHeal", type = "toggle",
				desc = "Toggles display on friendly targets.",
				get = function() return self.db.profile.showHeal end,
				set = function() self.db.profile.showHeal = not self.db.profile.showHeal self:TargetChanged() end,
			},
			combat = {
				name = "Combat", type = "toggle",
				desc = "Toggles combat status display. Will change text to 'OoC' when out of combat.",
				get = function() return self.db.profile.combat end,
				set = function( v ) self.db.profile.combat = not self.db.profile.combat self:TargetChanged() end,
			},

			
		},
	}
	self:RegisterDB("RangerDB")
	self:RegisterDefaults('profile', Default)
	if IsAddOnLoaded("RangerOptions") then self:RegisterChatCommand( {"/RangerCmd"}, Options ) else self:RegisterChatCommand( {"/Ranger"}, Options ) end
	rangerClass, _ = UnitClass("player")
end

function Ranger:GetValue(v)
	local rangerSetting = v
	local rangerValue = self.db.profile[rangerSetting]
	return rangerValue
end

function Ranger:SetValue(setting, v)
	local rangerSetting = setting
	self.db.profile[rangerSetting] = v
end

function Ranger:CreateFrameWork()	
	self.master = CreateFrame("Button", "RangerFrame", UIParent)
	self.master:Hide()
	self.master:SetMovable(true)
	self.master:EnableMouse(self.db.profile.drag and (not self.db.profile.lock))
	self.master:RegisterForDrag("LeftButton", "RightButton")
	self.master:SetScript("OnDragStart", function() if self.db.profile.drag and (not self.db.profile.lock) then self["master"]:StartMoving() rangerIsMoving = true end end)
	self.master:SetScript("OnDragStop", function() if rangerIsMoving then self["master"]:StopMovingOrSizing() self:SavePosition() rangerIsMoving = false end end)	
	self.master.texture = self.master:CreateTexture(nil, "DIALOG")
	self.master.Range = self.master:CreateFontString(nil, "OVERLAY") 
    self:Layout()
	
end

function Ranger:Layout()

	self.master:SetWidth(self.db.profile.size)
	self.master:SetHeight(self.db.profile.size)

	self.master:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 8,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
		edgeSize = 18,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	})
	
	self.master:SetBackdropBorderColor(0.5, 0.5, 0.5, self.db.profile.iconAlpha)
	self.master:SetBackdropColor(0, 0, 0, 0)
	
	local gameFont, _, _ = GameFontHighlightSmall:GetFont()
	self.master.Range:SetJustifyH("CENTER")
	self.master.Range:SetFont(gameFont, self.db.profile.fontSize, "OUTLINE") 
	self.master.Range:ClearAllPoints()
	self.master.Range:SetPoint("CENTER", self.master, "CENTER", self.db.profile.textXOffset, self.db.profile.textYOffset)
	self.master.Range:SetTextColor( 1,1,1 )
	self.master.Range:SetAlpha(self.db.profile.textAlpha)

	local anchoroffset = 5
	self.master.texture:SetPoint("TOPLEFT",self.master ,"TOPLEFT", anchoroffset, -anchoroffset)
	self.master.texture:SetPoint("TOPRIGHT",self.master ,"TOPRIGHT", -anchoroffset, -anchoroffset)
	self.master.texture:SetPoint("BOTTOMLEFT",self.master ,"BOTTOMLEFT", anchoroffset, anchoroffset)
	self.master.texture:SetPoint("BOTTOMRIGHT", self.master, "BOTTOMRIGHT", -anchoroffset, anchoroffset)
	self.master.texture:Show()
	self.master.texture:SetTexCoord(0.08,0.92,0.08,0.92);
	self.master.texture:SetAlpha(self.db.profile.iconAlpha)
	
	if self.db.profile.drag and (not self.db.profile.lock) then
		self.master:EnableMouse(true)
	else
		self.master:EnableMouse(false)
	end
	if self.db.profile.pos.x and self.db.profile.drag then
		local x = self.db.profile.pos.x
		local y = self.db.profile.pos.y

		local s = self.master:GetEffectiveScale()

		self.master:ClearAllPoints()
		self.master:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
	else
		self.master:ClearAllPoints()
		self.master:SetPoint(self.db.profile.anchorPoint, self.db.profile.anchor, self.db.profile.anchorTo, self.db.profile.xOffset, self.db.profile.yOffset)
	end	
end

function Ranger:SavePosition()

	local x, y = self.master:GetLeft(), self.master:GetTop()
	local s = self.master:GetEffectiveScale()
	
	self.db.profile.pos.x = x * s
	self.db.profile.pos.y = y * s

end

function Ranger:TargetChanged()
	if not self.db.profile.toggle then
		if rangerClass == "Warlock" then
			Ranger:Warlock() 
		elseif rangerClass == "Mage" then
			Ranger:Mage() 
		elseif rangerClass == "Priest" then
			Ranger:Priest() 
		elseif rangerClass == "Shaman" then
			Ranger:Shaman()
		elseif rangerClass == "Paladin"then
			Ranger:Paladin() 
		elseif rangerClass == "Hunter" then
			Ranger:Hunter()
		else
		end
	end
	if UnitExists("target") and (not UnitIsDead("target")) and UnitCanAttack("player", "target") and self.db.profile.showHarm and self.db.profile.toggle then
		self.index = nil
		self:ScheduleRepeatingEvent("Ranger", self.OnUpdate, 0.1, self)
		self.master:Show()		
	elseif UnitExists("target") and (not UnitIsGhost("target")) and UnitIsFriend("player", "target") and ((rangerClass == "Hunter" and UnitIsUnit("target", "pet")) or rangerClass ~= "Hunter") and self.db.profile.showHeal and self.db.profile.toggle then
		self.index = nil
		self:ScheduleRepeatingEvent("Ranger", self.OnUpdate, 0.1, self)
		self.master:Show()	
	elseif UnitExists("target") and (not UnitIsDead("target")) and UnitCanAttack("player", "target") and self.db.profile.showHarm and (not self.db.profile.toggle) then
		self.index = nil
		self:ScheduleRepeatingEvent("Ranger", self.OnUpdate, 0.1, self)
		self.master:Show()
	elseif UnitExists("target") and UnitIsFriend("target", "player") and (not UnitIsGhost("target")) and ((rangerClass == "Priest" or rangerClass == "Paladin" or rangerClass == "Shaman" or rangerClass == "Druid" or (rangerClass == "Mage" and (not UnitIsDead("target"))) or (rangerClass == "Hunter" and UnitIsUnit("target", "pet")))) and self.db.profile.showHeal and (not self.db.profile.toggle) then
		self.index = nil
		self:ScheduleRepeatingEvent("Ranger", self.OnUpdate, 0.1, self)
		self.master:Show()
	else
		self:CancelScheduledEvent("Ranger")
		self.master:Hide()
	end
	Ranger:OnUpdate()
end

function Ranger:OnUpdate()
	if self.db.profile.toggle then
		if UnitCanAttack("player", "target") then
			Ranger:AdvancedUpdateHarm()
		else
			Ranger:AdvancedUpdateHeal()
		end
	elseif rangerClass == "Priest" then
		if UnitCanAttack("player", "target") then
			Ranger:PriestUpdateHarm()
		else
			Ranger:PriestUpdateHeal()
		end
	elseif rangerClass == "Paladin" then
		if UnitCanAttack("player", "target") then
			Ranger:PaladinUpdateHarm()
		else
			Ranger:PaladinUpdateHeal()
		end
	elseif rangerClass == "Shaman" then
		if UnitCanAttack("player", "target") then
			Ranger:ShamanUpdateHarm()
		else
			Ranger:ShamanUpdateHeal()
		end
	elseif rangerClass == "Druid" then
		if UnitCanAttack("player", "target") then
			Ranger:DruidUpdateHarm()
		else
			Ranger:DruidUpdateHeal()
		end
	elseif rangerClass == "Mage" then
		if UnitCanAttack("player", "target") then
			Ranger:MageUpdateHarm()
		else
			Ranger:MageUpdateHeal()
		end
	elseif rangerClass == "Hunter" then
		if UnitCanAttack("player", "target") then
			Ranger:HunterUpdateHarm()
		else
			Ranger:HunterUpdateHeal()
		end
	elseif rangerClass == "Warlock" then
		Ranger:WarlockUpdateHarm()
	elseif rangerClass == "Rogue" then
		Ranger:RogueUpdateHarm()
	elseif rangerClass == "Warrior" then
		Ranger:WarriorUpdateHarm()
	else
		return
	end					
end

function Ranger:SetRange(rangerRange, rangerTexture, index)
	if rangerIndex ~= rangerRange or (not rangerRange) then
		rangerText = rangerRange
		rangerIndex = rangerRange
		if index and index == 0 then
			self.master.texture:SetTexture("Interface\\Icons\\"..rangerTexture)
		elseif index then
			self.master.texture:SetTexture(GetActionTexture(index))
		elseif rangerTexture then
			self.master.texture:SetTexture("Interface\\Icons\\"..rangerTexture)
		else
		end
	else end
	if self.db.profile.combat and (not rangerCombat) then
		rangerText = "OoC"
	end
end

function Ranger:Usable(rangerSpellName)
	rangerIsUsable, _ = IsUsableSpell(rangerSpellName)
	return rangerIsUsable
end

function Ranger:AdvancedUpdateHarm()
	if self.db.profile.close and CheckInteractDistance("target", 3) == 1 then
		if self.db.profile.closeTexture then
			Ranger:SetRange("10", self.db.profile.closeTexture)
		else
			Ranger:SetRange("10", "Spell_Nature_FaerieFire")
		end
	elseif self.db.profile.range1 ~= 0 and IsActionInRange(self.db.profile.range1) == 1 then
		if not self.db.profile.range1Texture then
			Ranger:SetRange(self.db.profile.range1Text, nil, self.db.profile.range1)
		else
			Ranger:SetRange(self.db.profile.range1Text, self.db.profile.range1Texture)
		end
	elseif self.db.profile.range2 ~= 0 and IsActionInRange(self.db.profile.range2) == 1 then
		if not self.db.profile.range2Texture then
			Ranger:SetRange(self.db.profile.range2Text, nil, self.db.profile.range2)
		else
			Ranger:SetRange(self.db.profile.range2Text, self.db.profile.range2Texture)
		end
	elseif self.db.profile.range3 ~= 0 and IsActionInRange(self.db.profile.range3) == 1 then
		if not self.db.profile.range3Texture then
			Ranger:SetRange(self.db.profile.range3Text, nil, self.db.profile.range3)
		else
			Ranger:SetRange(self.db.profile.range3Text, self.db.profile.range3Texture)
		end
	elseif self.db.profile.range4 ~= 0 and IsActionInRange(self.db.profile.range4) == 1 then
		if not self.db.profile.range4Texture then
			Ranger:SetRange(self.db.profile.range4Text, nil, self.db.profile.range4)
		else
			Ranger:SetRange(self.db.profile.range4Text, self.db.profile.range4Texture)
		end
	elseif self.db.profile.range5 ~= 0 and IsActionInRange(self.db.profile.range5) == 1 then
		if not self.db.profile.range5Texture then
			Ranger:SetRange(self.db.profile.range5Text, nil, self.db.profile.range5)
		else
			Ranger:SetRange(self.db.profile.range5Text, self.db.profile.range5Texture)
		end
	else
		if not self.db.profile.oorTexture then
			Ranger:SetRange("OoR", "Spell_Nature_Tranquility", 0)
		else
			Ranger:SetRange("OoR", self.db.profile.oorTexture)
		end
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:AdvancedUpdateHeal()
	if self.db.profile.close and CheckInteractDistance("target", 3) == 1 then
		if self.db.profile.closeTexture then
			Ranger:SetRange("10", self.db.profile.closeTexture)
		else
			Ranger:SetRange("10", "Spell_Nature_FaerieFire", 0)
		end
	elseif self.db.profile.heal1 ~= 0 and IsActionInRange(self.db.profile.heal1) == 1 then
		if not self.db.profile.heal1Texture then
			Ranger:SetRange(self.db.profile.heal1Text, nil, self.db.profile.heal1)
		else
			Ranger:SetRange(self.db.profile.heal1Text, self.db.profile.heal1Texture)
		end
    elseif self.db.profile.heal2 ~= 0 and IsActionInRange(self.db.profile.heal2) == 1 then
		if not self.db.profile.heal2Texture then
			Ranger:SetRange(self.db.profile.heal2Text, nil, self.db.profile.heal2)
		else
			Ranger:SetRange(self.db.profile.heal2Text, self.db.profile.heal2Texture)
		end
	else
		if not self.db.profile.oorTexture then
			Ranger:SetRange("OoR", "Spell_Nature_Tranquility", 0)
		else
			Ranger:SetRange("OoR", self.db.profile.oorTexture)
		end
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:Warlock()
	local _, _, _, _, rangerRankDestro, _, _, _ = GetTalentInfo(3, 10)
	local _, _, _, _, rangerRankAffli, _, _, _ = GetTalentInfo(1, 10)
	_, _, _, _, rangerRankSB, _, _, _ = GetTalentInfo(3, 8)
	if rangerRankDestro == 0 then
		rangerShadowburnRange = 20
		rangerShadowboltRange = 30
	elseif rankDestro == 1 then
		rangerShadowburnRange = 22
		rangerShadowboltRange = 33
	else
		rangerShadowburnRange = 24
		rangerShadowboltRange = 36
	end
	if rangerRankAffli == 0 then
		rangerFearRange = 20
		rangerCorruptionRange = 30
	elseif rangerRankAffli == 1 then
		rangerFearRange = 22
		rangerCorruptionRange = 33
	else
		rangerFearRange = 24
		rangerCorruptionRange = 36
	end
end

function Ranger:WarlockUpdateHarm()
	if rangerShadowburnRange >= rangerFearRange and IsSpellInRange("Fear") == 1 then
		Ranger:SetRange(rangerFearRange, "Spell_Shadow_Possession")
	elseif rangerShadowburnRange >= rangerFearRange and IsSpellInRange("Shadowburn") == 1 and rangerRankSB == 1 then
		Ranger:SetRange(rangerShadowburnRange, "Spell_Shadow_ScourgeBuild")
	elseif rangerShadowburnRange < rangerFearRange and IsSpellInRange("Shadowburn") == 1 and rangerRankSB == 1 then
		Ranger:SetRange(rangerShadowburnRange, "Spell_Shadow_ScourgeBuild")
	elseif rangerShadowburnRange < rangerFearRange and IsSpellInRange("Fear") == 1 then
		Ranger:SetRange(rangerFearRange, "Spell_Shadow_Possession")

	elseif rangerShadowboltRange >= rangerCorruptionRange and IsSpellInRange("Corruption") == 1 then
		Ranger:SetRange(rangerCorruptionRange, "Spell_Shadow_AbominationExplosion")
	elseif rangerShadowboltRange >= rangerCorruptionRange and IsSpellInRange("Shadow Bolt") == 1 then
		Ranger:SetRange(rangerShadowboltRange, "Spell_Shadow_ShadowBolt")
	elseif rangerShadowboltRange < rangerCorruptionRange and IsSpellInRange("Shadow Bolt") == 1 then
		Ranger:SetRange(rangerShadowboltRange, "Spell_Shadow_ShadowBolt")
	elseif rangerShadowboltRange < rangerCorruptionRange and IsSpellInRange("Corruption") == 1 then
		Ranger:SetRange(rangerCorruptionRange, "Spell_Shadow_AbominationExplosion")

	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end

	Ranger.master.Range:SetText( rangerText )
end

function Ranger:Mage()
	local itemLink = GetInventoryItemLink("player",GetInventorySlotInfo("HandsSlot"))
	local found, _, itemString = string.find(itemLink, "^|c%x+|H(.+)|h%[.+%]")
	local found, _, itemId, _, _, _, _, _, _, _ = string.find(itemString, "^item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)")
	if itemId == 25857 or itemId == 28868 then 
		rangerMageGloves = true
	else
		rangerMageGloves = false
	end
	_, _, _, _, rangerRankFire, _, _, _ = GetTalentInfo(2, 4)
	_, _, _, _, rangerRankFrost, _, _, _ = GetTalentInfo(3, 11)
	if rangerRankFire == 0 then
		if rangerMageGloves then
			rangerFireblastRange = 25
		else
			rangerFireblastRange = 20
		end
		rangerScorchRange = 30
		rangerFireballRange = 35
	elseif rangerRankFire == 1 then
		if rangerMageGloves then
			rangerFireblastRange = 28
		else
			rangerFireblastRange = 23
		end
		rangerScorchRange = 33
		rangerFireballRange = 35
	else
		if rangerMageGloves then
			rangerFireblastRange = 31
		else
			rangerFireblastRange = 26
		end
		rangerScorchRange = 36
		rangerFireballRange = 41
	end
	if rangerRankFrost == 0 then
		rangerFrostboltRange = 30
	elseif rangerRankFrost == 1 then
		rangerFrostboltRange = 33
	else
		rangerFrostboltRange = 36
	end
	rangerArcaneRange = 30
end

function Ranger:MageUpdateHarm()
	if IsSpellInRange("Fire Blast") == 1 and rangerFireblastRange < 30 then
		Ranger:SetRange(rangerFireblastRange, "Spell_Fire_Fireball")
	elseif rangerRankFire > 0 and rangerRankFrost > 0 and IsSpellInRange("Arcane Missiles") then
		Ranger:SetRange(rangerArcaneRange, "Spell_Nature_StarFall")
	elseif rangerRankFire > 0 and rangerRankFrost > 0 and IsSpellInRange("Fire Blast") then
		Ranger:SetRange(rangerFireblastRange,"Spell_Fire_Fireball")
	elseif rangerFrostboltRange >= rangerScorchRange and IsSpellInRange("Scorch") == 1 then
		Ranger:SetRange(rangerRankScorch, "Spell_Fire_SoulBurn")
	elseif rangerFrostboltRange >= rangerScorchRange and rangerFrostboltRange < rangerFireballRange and IsSpellInRange("Frostbolt") == 1 then
		Ranger:SetRange(rangerFrostboltRange, "Spell_Frost_FrostBolt02")
	elseif rangerFrostboltRange > rangerFireballRange and IsSpellInRange("Fireball") == 1 then
		Ranger:SetRange(rangerFireballRange, "Spell_Fire_FlameBolt")
	elseif rangerFrostboltRange >= rangerFireballRange and IsSpellInRange("Frostbolt") == 1 then
		Ranger:SetRange(rangerFrostboltRange, "Spell_Frost_FrostBolt02")
	elseif rangerFrostboltRange < rangerScorchRange and rangerFrostboltRange < rangerFireblastRange and IsSpellInRange("Frostbolt") == 1 then
		Ranger:SetRange(rangerFrostboltRange, "Spell_Frost_FrostBolt02")
	elseif rangerFrostboltRange < rangerScorchRange and rangerFrostboltRange < rangerFireblastRange and IsSpellInRange("Fire Blast") == 1 then
		Ranger:SetRange(rangerFireblastRange, "Spell_Fire_Fireball")
	elseif rangerFrostboltRange < rangerScorchRange and IsSpellInRange("Frostbolt") == 1 then
		Ranger:SetRange(rangerFrostboltRange, "Spell_Frost_FrostBolt02")
	elseif rangerFrostboltRange < rangerScorchRange and IsSpellInRange("Scorch") == 1 then
		Ranger:SetRange(rangerScorchRange, "Spell_Fire_SoulBurn")
	elseif rangerFrostboltRange < rangerFireballRange and IsSpellInRange("Fireball") == 1 then
		Ranger:SetRange(rangerFireballRange, "Spell_Fire_FlameBolt")

	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end

	Ranger.master.Range:SetText( rangerText )
end

function Ranger:MageUpdateHeal()
	if IsSpellInRange("Arcane Intelligence") then
		Ranger:SetRange("30", "Spell_Nature_RemoveCurse")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
end

function Ranger:RogueGet()
	local _, _, _, _, rangerRankPremed, _, _, _ = GetTalentInfo(3, 19)
	return rangerRankPremed
end

function Ranger:RogueUpdateHarm()
	if IsSpellInRange("Sinister Strike") == 1 then
		Ranger:SetRange("Melee", "Spell_Shadow_RitualOfSacrifice")
	elseif IsSpellInRange("Blind") == 1 and (IsSpellInRange("Shoot Bow") or IsSpellInRange("Shoot Crossbow") or IsSpellInRange("Shoot Gun") or IsSpellInRange("Throw")) == 1 then
		Ranger:SetRange("8-10", "Spell_Shadow_MindSteal")
	elseif IsSpellInRange("Blind") == 1 then
		Ranger:SetRange("5-8", "Spell_Nature_Drowsy")
	elseif IsSpellInRange("Premeditation") == 1 and Ranger:RogueGet() == 1 then
		Ranger:SetRange("20", "Spell_Shadow_Possession")
	elseif (IsSpellInRange("Shoot Bow") or IsSpellInRange("Shoot Crossbow") or IsSpellInRange("Shoot Gun") or IsSpellInRange("Throw")) == 1 then
		Ranger:SetRange("30", "Ability_Marksmanship")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:WarriorUpdateHarm()
	if IsSpellInRange("Hamstring") == 1 then
		Ranger:SetRange("Melee", "Ability_ShockWave")
	elseif IsSpellInRange("Intimidating Shout") == 1 and IsSpellInRange("Charge") == 1 and (not rangerCombat) then
		Ranger:SetRange("8-10", "Ability_Warrior_Charge")
	elseif IsSpellInRange("Intimidating Shout") == 1 and IsSpellInRange("Intercept") == 1 then
		Ranger:SetRange("8-10", "Ability_GolemThunderClap")
	elseif IsSpellInRange("Charge") == 1 and (not rangerCombat) then
		Ranger:SetRange("10-25", "Ability_Warrior_Charge")
	elseif IsSpellInRange("Intercept") == 1 then
		Ranger:SetRange("10-25", "Ability_Rogue_Sprint")
	elseif IsSpellInRange("Intimidating Shout") == 1 then
		Ranger:SetRange("5-8", "Spell_Nature_Drowsy")
	elseif (IsSpellInRange("Shoot Bow") or IsSpellInRange("Shoot Crossbow") or IsSpellInRange("Shoot Gun") or IsSpellInRange("Throw")) == 1 then
		Ranger:SetRange("30", "Ability_Marksmanship")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:Priest()
	local _, _, _, _, rangerRankShadow, _, _, _ = GetTalentInfo(3, 10)
	local _, _, _, _, rangerRankHoly, _, _, _ = GetTalentInfo(2, 9)
	_, _, _, _, rangerRankFlay, _, _, _ = GetTalentInfo(3, 8)
	_, _, _, _, rangerRankNova, _, _, _ = GetTalentInfo(2, 6)
	if rangerRankShadow == 0 then
		rangerFlayRange = 20
		rangerMbRange = 30
		rangerSootheRange = 40
	elseif rangerRankShadow == 1 then
		rangerFlayRange = 22
		rangerMbRange = 33
		rangerSootheRange = 44
	else
		rangerFlayRange = 24
		rangerMbRange = 36
		rangerSootheRange = 48
	end
	if rangerRankHoly == 0 then
		rangerSmiteRange = 30
	elseif rangerRankHoly == 1 then
		rangerSmiteRange = 33
	else
		rangerSmiteRange = 36
	end
end

function Ranger:PriestUpdateHarm()
	if CheckInteractDistance("target", 3) and rangerRankNova == 1 then
		Ranger:SetRange("10", "Spell_Holy_HolyNova")
	elseif IsSpellInRange("Mind Control") == 1 then
		Ranger:SetRange("20", "Spell_Shadow_UnholyFrenzy")
	elseif rangerRankFlay == 1 and IsSpellInRange("Mind Flay") == 1 then
		Ranger:SetRange(rangerFlayRange, "Spell_Shadow_SiphonMana")
	elseif rangerSmiteRange > 30 and rangerMbRange > 30 then
		Ranger:SetRange("30", "Spell_Holy_DispelMagic")
	elseif IsSpellInRange("Mind Blast") == 1 and rangerSmiteRange >= rangerMbRange then
		Ranger:SetRange(rangerMbRange, "Spell_Shadow_ShadowWordPain")
	elseif IsSpellInRange("Smite") == 1 and rangerSmiteRange >= rangerMbRange then
		Ranger:SetRange(rangerSmiteRange, "Spell_Holy_HolySmite")
	elseif  IsSpellInRange("Smite") == 1 and rangerSmiteRange < rangerMbRange then
		Ranger:SetRange(rangerSmiteRange, "Spell_Holy_SearingLight")
	elseif IsSpellInRange("Mind Blast") == 1 and rangerSmiteRange < rangerMbRange then
		Ranger:SetRange(rangerMbRange, "Spell_Shadow_ShadowWordPain")
	elseif IsSpellInRange("Mind Soothe") == 1 and Ranger:Usable("Mind Soothe") then
		Ranger:SetRange(rangerSootheRange, "Spell_Holy_MindSooth")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:PriestUpdateHeal()
	if CheckInteractDistance("target", 3) and rangerRankNova == 1 then
		Ranger:SetRange("10", "Spell_Holy_HolyNova")
	elseif IsSpellInRange("Power Word: Fortitude") == 1 then
		Ranger:SetRange("30", "Spell_Holy_DispelMagic")
	elseif IsSpellInRange("Flash Heal") == 1 then
		Ranger:SetRange("40", "Spell_Holy_FlashHeal")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:DruidGet()
	local _, _, _, _, rangerRankCharge, _, _, _ = GetTalentInfo(2, 7)
	return rangerRankCharge
end

function Ranger:DruidUpdateHarm()
	if IsSpellInRange("Bash") == 1 then
		Ranger:SetRange("Melee", "Ability_Druid_Bash")
	elseif CheckInteractDistance("target", 1) == 1 and Ranger:DruidGet() == 1 and IsSpellInRange("Feral Charge") == 0 then
		Ranger:SetRange("5-8", "Ability_Hunter_Pet_Bear")
	elseif Ranger:DruidGet() == 1 and IsSpellInRange("Feral Charge") == 1 then
		Ranger:SetRange("8-25", "Ability_Hunter_Pet_Bear")
	elseif IsSpellInRange("Moonfire") == 1 then
		Ranger:SetRange("30", "Spell_Nature_StarFall")
	elseif IsSpellInRange("Soothe Animal") == 1 and Ranger:Usable("Soothe Animal") == 1 then
		Ranger:SetRange("40", "Ability_Hunter_BeastSoothe")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:DruidUpdateHeal()
	if IsSpellInRange("Thorns") == 1 then
		Ranger:SetRange("30", "Spell_Nature_Thorns")
	elseif IsSpellInRange("Rejuvenation") == 1 then
		Ranger:SetRange("40", "Spell_Nature_Rejuvenation")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:ShamanGet()
	local _, _, _, _, rankStorm, _, _, _ = GetTalentInfo(2, 19)
	return rankStorm
end

function Ranger:Shaman()
	local itemLink = GetInventoryItemLink("player",GetInventorySlotInfo("HandsSlot"))
	local found, _, itemString = string.find(itemLink, "^|c%x+|H(.+)|h%[.+%]")
	local found, _, itemId, _, _, _, _, _, _, _ = string.find(itemString, "^item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)")
	if itemId == 27470 or itemId == 28847 then 
		rangerShockRange = 25
	else
		rangerShockRange = 20
	end
	local _, _, _, _, rangerRankReach, _, _, _ = GetTalentInfo(1, 12)
	if rangerRankReach == 0 then
		rangerLbRange = 30
	elseif rangerRankReach == 1 then
		rangerLbRange = 33
	else
		rangerLbRange = 36
	end
end

function Ranger:ShamanUpdateHarm()
	if Ranger:ShamanGet() == 1 and IsSpellInRange("Stormstrike") == 1 then
		Ranger:SetRange("Melee", "Spell_Holy_SealOfMight")
	elseif IsSpellInRange("Earth Shock") == 1 then
		Ranger:SetRange(rangerShockRange, "Spell_Nature_EarthShock")
	elseif rangerLbRange > 30 and IsSpellInRange("Purge") == 1 then
		Ranger:SetRange("30", "Spell_Nature_Purge")
	elseif IsSpellInRange("Lightning Bolt") == 1 then
		Ranger:SetRange(rangerLbRange, "Spell_Nature_Lightning")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:ShamanUpdateHeal()
	if IsSpellInRange("Water Breathing") == 1 then
		Ranger:SetRange("30", "Spell_Nature_NullifyPoison")
	elseif IsSpellInRange("Healing Wave") == 1 then
		Ranger:SetRange("40", "Spell_Nature_MagicImmunity")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:Paladin()
	_, _, _, _, rangerRankShield, _, _, _ = GetTalentInfo(2, 21)
	_, _, _, _, rangerRankStrike, _, _, _ = GetTalentInfo(3, 22)
	_, _, _, _, rangerRankRepentance, _, _, _ = GetTalentInfo(3, 19)
	_, _, _, _, rangerRankShock, _, _, _ = GetTalentInfo(1, 17)
end

function Ranger:PaladinUpdateHarm()
	if rangerRankStrike == 1 and IsSpellInRange("Crusader Strike") == 1 then
		Ranger:SetRange("Melee", "INV_Sword_49")
	elseif rangerRankShield == 1 and IsSpellInRange("Avenger's Shield") == 1 then
		Ranger:SetRange("8-30", "Spell_Holy_AvengersShield")
	elseif CheckInteractDistance("target", 3) == 1 then
		Ranger:SetRange("10", "Spell_Holy_RighteousFury")
	elseif rangerRankRepentance == 1 and IsSpellInRange("Repentance") == 1 then
		Ranger:SetRange("20", "Spell_Holy_PrayerOfHealing")
	elseif rangerRankShock == 1 and IsSpellInRange("Holy Shock") == 1 then
		Ranger:SetRange("20", "Spell_Holy_SearingLight")
	elseif IsSpellInRange("Exorcism") == 1 and Ranger:Usable("Exorcism") == 1 then
		Ranger:SetRange("30", "Spell_Holy_Excorcism_02")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:PaladinUpdateHeal()
	if rangerRankShock == 1 and IsSpellInRange("Holy Shock") == 1 then
		Ranger:SetRange("20", "Spell_Holy_SearingLight")
	elseif IsSpellInRange("Blessing of Might") == 1 then
		Ranger:SetRange("30", "Spell_Holy_Renew")
	elseif IsSpellInRange("Holy Light") == 1 then
		Ranger:SetRange("40", "Spell_Holy_HolyBolt")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:Hunter()
	local _, _, _, _, rangerRankHawk, _, _, _ = GetTalentInfo(3, 3)
	_, _, _, _, rangerRankScatter, _, _, _ = GetTalentInfo(2, 12)
	if rangerRankHawk == 0 then
		rangerAutoRange = 35
		rangerScatterRange = 15
	elseif rangerRankHawk == 1 then
		rangerAutoRange = 37
		rangerScatterRange = 17
	elseif rangerRankHawk == 2 then
		rangerAutoRange = 39
		rangerScatterRange = 19
	else
		rangerAutoRange = 41
		rangerScatterRange = 21
	end
end
	
function Ranger:HunterUpdateHarm()
	if IsSpellInRange("Wing Clip") == 1 then
		Ranger:SetRange("Melee", "Ability_Rogue_Trip")
	elseif CheckInteractDistance("target", 4) == 1 and IsSpellInRange("Auto Shot") ~= 1 then
		Ranger:SetRange("5-8", "Spell_Nature_Drowsy")
	elseif IsSpellInRange("Scare Beast") == 1 and Ranger:Usable("Scare Beast") then
		Ranger:SetRange("10", "Ability_Druid_Cower")
	elseif rangerRankScatter == 1 and IsSpellInRange("Scatter Shot") == 1 then
		Ranger:SetRange(rangerScatterRange, "Ability_GolemStormBolt")
	elseif IsSpellInRange("Auto Shot") == 1 then
		Ranger:SetRange(rangerAutoRange, "Ability_Marksmanship")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end

function Ranger:HunterUpdateHeal()
	if IsSpellInRange("Mend Pet") == 1 then
		Ranger:SetRange("45", "Ability_Hunter_MendPet")
	else
		Ranger:SetRange("OoR", "Spell_Nature_Tranquility")
	end
	Ranger.master.Range:SetText( rangerText )
end